@@ -1,9 +1,12 @@
 #include "TrackSegment.h"
+
+#include "Global.h"
 #include "DocumentCommands.h"
 #include "TrackSegmentCommands.h"
 #include "MapView.h"
+#include "MapRenderer.h"
 #include "Node.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
 
 #include <QtGui/QPainter>
 #include <QProgressDialog>
@@ -11,6 +14,8 @@
 #include <algorithm>
 #include <QList>
 
+#define TEST_RFLAGS_TRACK_SEGMENT(x) theView->renderOptions().options.testFlag(x)
+
 class TrackSegmentPrivate
 {
     public:
@@ -19,20 +24,21 @@ class TrackSegmentPrivate
         {
         }
 
-        QList<Node*> Nodes;
-        double Distance;
+        QList<TrackNode*> Nodes;
+        qreal Distance;
+        CoordBox BBox;
 };
 
 TrackSegment::TrackSegment(void)
+    : Feature()
 {
     p = new TrackSegmentPrivate;
-    setRenderPriority(RenderPriority (RenderPriority::IsLinear,0.,99));
 }
 
 TrackSegment::TrackSegment(const TrackSegment& other)
-: Feature(other), p(0)
+    : Feature(other)
 {
-    setRenderPriority(RenderPriority (RenderPriority::IsLinear,0.,99));
+    p = new TrackSegmentPrivate;
 }
 
 TrackSegment::~TrackSegment(void)
@@ -65,16 +71,18 @@ QString TrackSegment::description() const
     return "tracksegment";
 }
 
-void TrackSegment::add(Node* aPoint)
+void TrackSegment::add(TrackNode* aPoint)
 {
     p->Nodes.push_back(aPoint);
     aPoint->setParentFeature(this);
+    g_backend.sync(this);
 }
 
-void TrackSegment::add(Node* Pt, int Idx)
+void TrackSegment::add(TrackNode* Pt, int Idx)
 {
     p->Nodes.push_back(Pt);
     std::rotate(p->Nodes.begin()+Idx,p->Nodes.end()-1,p->Nodes.end());
+    g_backend.sync(this);
 }
 
 int TrackSegment::find(Feature* Pt) const
@@ -90,6 +98,7 @@ void TrackSegment::remove(int idx)
     Node* Pt = p->Nodes[idx];
     p->Nodes.erase(p->Nodes.begin()+idx);
     Pt->unsetParentFeature(this);
+    g_backend.sync(this);
 }
 
 void TrackSegment::remove(Feature* F)
@@ -109,7 +118,7 @@ Feature* TrackSegment::get(int i)
     return p->Nodes[i];
 }
 
-Node* TrackSegment::getNode(int i)
+TrackNode* TrackSegment::getNode(int i)
 {
     return p->Nodes[i];
 }
@@ -129,9 +138,9 @@ void TrackSegment::drawDirectionMarkers(QPainter &P, QPen &pen, const QPointF &
     if (::distance(FromF,ToF) <= 30.0)
         return;
 
-    const double DistFromCenter=10.0;
-    const double theWidth = !M_PREFS->getSimpleGpxTrack() ? 5.0 : 8.0;
-    const double A = angle(FromF-ToF);
+    const qreal DistFromCenter=10.0;
+    const qreal theWidth = !M_PREFS->getSimpleGpxTrack() ? 5.0 : 8.0;
+    const qreal A = angle(FromF-ToF);
 
     QPointF T(DistFromCenter*cos(A), DistFromCenter*sin(A));
     QPointF V1(theWidth*cos(A+M_PI/6),theWidth*sin(A+M_PI/6));
@@ -140,34 +149,38 @@ void TrackSegment::drawDirectionMarkers(QPainter &P, QPen &pen, const QPointF &
     pen.setStyle(Qt::SolidLine);
     P.setPen(pen);
 
-    QPointF H((FromF+ToF) / 2.0);
+    QPointF H(FromF+ToF);
+    H /= 2.0;
     P.drawLine(H-T,H-T+V1);
     P.drawLine(H-T,H-T+V2);
 }
 
-void TrackSegment::draw(QPainter &P, MapView* theView)
+void TrackSegment::drawSimple(QPainter &P, MapView *theView)
+{
+    Q_UNUSED(P)
+    Q_UNUSED(theView)
+}
+
+void TrackSegment::drawTouchup(QPainter &P, MapView* theView)
 {
     QPen pen;
 
-    if (!M_PREFS->getTrackSegmentsVisible())
+    if (!TEST_RFLAGS_TRACK_SEGMENT(RendererOptions::TrackSegmentVisible))
         return;
 
     for (int i=1; i<p->Nodes.size(); ++i)
     {
-        Coord last = p->Nodes[i-1]->position();
-        Coord here = p->Nodes[i]->position();
-
-        if (CoordBox::visibleLine(theView->viewport(), last, here) == false)
+        if (!theView->viewport().contains(p->Nodes[i-1]->position()) && !theView->viewport().contains(p->Nodes[i]->position()))
             continue;
 
-        QPointF FromF(theView->toView(last));
-        QPointF ToF(theView->toView(here));
+        QPointF FromF = theView->toView(p->Nodes[i-1]);
+        QPointF ToF = theView->toView(p->Nodes[i]);
 
         if (!M_PREFS->getSimpleGpxTrack())
         {
-            double distance = here.distanceFrom(last);
-            double slope = (p->Nodes[i]->elevation() - p->Nodes[i-1]->elevation()) / (distance * 10.0);
-            double speed = p->Nodes[i]->speed();
+            qreal distance = p->Nodes[i-1]->position().distanceFrom(p->Nodes[i]->position());
+            qreal slope = (p->Nodes[i]->elevation() - p->Nodes[i-1]->elevation()) / (distance * 10.0);
+            qreal speed = p->Nodes[i]->speed();
 
             int width = M_PREFS->getGpxTrackWidth();
             // Dynamic track line width adaption to zoom level
@@ -177,7 +190,7 @@ void TrackSegment::draw(QPainter &P, MapView* theView)
                 width--;
 
             // Encode speed in width of path ...
-            double penWidth = 1.0;
+            qreal penWidth = 1.0;
             if (speed > 10.0)
                 penWidth = qMin(1.0+speed*0.02, 5.0);
 
@@ -189,12 +202,12 @@ void TrackSegment::draw(QPainter &P, MapView* theView)
 
             if (slope > 2.0)
             {
-                slope = qMin(slope, 20.0);
+                slope = qMin(slope, (qreal)20.0);
                 green = 48 + int(slope*79.0 / 20.0);
             }
             else if (slope < -2.0)
             {
-                slope = qMax(slope, - 20.0);
+                slope = qMax(slope, (qreal)-20.0);
                 red = 48 + int(-slope*79.0 / 20.0);
             }
 
@@ -226,34 +239,35 @@ bool TrackSegment::notEverythingDownloaded()
     return false;
 }
 
-void TrackSegment::drawFocus(QPainter &, MapView*, bool)
+void TrackSegment::drawSpecial(QPainter&, QPen&, MapView*)
 {
-    // Can't be selection
+    // not implemented
 }
 
-void TrackSegment::drawHover(QPainter &, MapView*, bool)
+void TrackSegment::drawParentsSpecial(QPainter&, QPen&, MapView*)
 {
-    // Can't be highlighted
+    // not implemented
 }
 
-void TrackSegment::drawHighlight(QPainter &, MapView*, bool)
+void TrackSegment::drawChildrenSpecial(QPainter&, QPen&, MapView*, int)
 {
-    // Can't be selection
+    // not implemented
 }
 
-CoordBox TrackSegment::boundingBox() const
+
+const CoordBox& TrackSegment::boundingBox(bool) const
 {
     if (p->Nodes.size())
     {
-        CoordBox Box(p->Nodes[0]->position(),p->Nodes[0]->position());
+        p->BBox = CoordBox(p->Nodes[0]->position(),p->Nodes[0]->position());
         for (int i=1; i<p->Nodes.size(); ++i)
-            Box.merge(p->Nodes[i]->position());
-        return Box;
-    }
-    return CoordBox(Coord(0,0),Coord(0,0));
+            p->BBox.merge(p->Nodes[i]->position());
+    } else
+        p->BBox = CoordBox();
+    return p->BBox;
 }
 
-double TrackSegment::pixelDistance(const QPointF& , double , bool , MapView*) const
+qreal TrackSegment::pixelDistance(const QPointF& , qreal , const QList<Feature*>& , MapView*) const
 {
     // unable to select that one
     return 1000000;
@@ -264,10 +278,10 @@ void TrackSegment::cascadedRemoveIfUsing(Document* theDocument, Feature* aFeatur
     for (int i=0; i<p->Nodes.size();) {
         if (p->Nodes[i] == aFeature)
         {
-            QList<Node*> Alternatives;
+            QList<TrackNode*> Alternatives;
             for (int j=0; j<Proposals.size(); ++j)
             {
-                Node* Pt = dynamic_cast<Node*>(Proposals[j]);
+                TrackNode* Pt = CAST_TRACKNODE(Proposals[j]);
                 if (Pt)
                     Alternatives.push_back(Pt);
             }
@@ -288,7 +302,7 @@ void TrackSegment::cascadedRemoveIfUsing(Document* theDocument, Feature* aFeatur
                         }
                     }
                 }
-                theList->add(new TrackSegmentRemoveNodeCommand(this, (Node*)aFeature,aFeature->layer()));
+                theList->add(new TrackSegmentRemoveNodeCommand(this, (TrackNode*)aFeature,aFeature->layer()));
             }
         }
         ++i;
@@ -301,6 +315,12 @@ void TrackSegment::partChanged(Feature*, int)
 
 void TrackSegment::updateMeta()
 {
+    QMutexLocker mutlock(&featMutex);
+    if (MetaUpToDate)
+        return;
+
+    Feature::updateMeta();
+
     p->Distance = 0;
 
     if (p->Nodes.size() == 0)
@@ -323,7 +343,7 @@ void TrackSegment::updateMeta()
     MetaUpToDate = true;
 }
 
-double TrackSegment::distance()
+qreal TrackSegment::distance()
 {
     if (MetaUpToDate == false)
         updateMeta();
@@ -337,19 +357,19 @@ int TrackSegment::duration() const
 }
 
 
-bool TrackSegment::toGPX(QDomElement xParent, QProgressDialog * progress, bool forExport)
+bool TrackSegment::toGPX(QXmlStreamWriter& stream, QProgressDialog * progress, bool forExport)
 {
     bool OK = true;
 
-    QDomElement e = xParent.ownerDocument().createElement("trkseg");
-    xParent.appendChild(e);
+    stream.writeStartElement("trkseg");
 
     if (!forExport)
-        e.setAttribute("xml:id", xmlId());
+        stream.writeAttribute("xml:id", xmlId());
 
     for (int i=0; i<size(); ++i) {
-        dynamic_cast <Node*> (get(i))->toGPX(e, progress, forExport);
+        dynamic_cast <Node*> (get(i))->toGPX(stream, progress, "trkpt", forExport);
     }
+    stream.writeEndElement();
 
     if (progress)
         progress->setValue(progress->value()+1);
@@ -357,36 +377,37 @@ bool TrackSegment::toGPX(QDomElement xParent, QProgressDialog * progress, bool f
     return OK;
 }
 
-bool TrackSegment::toXML(QDomElement xParent, QProgressDialog * progress, bool)
+bool TrackSegment::toXML(QXmlStreamWriter& stream, QProgressDialog * progress, bool, QString)
 {
-    return toGPX(xParent, progress, false);
+    return toGPX(stream, progress, false);
 }
 
-TrackSegment* TrackSegment::fromGPX(Document* d, Layer* L, const QDomElement e, QProgressDialog * progress)
+TrackSegment* TrackSegment::fromGPX(Document* d, Layer* L, QXmlStreamReader& stream, QProgressDialog * progress)
 {
-    TrackSegment* l = new TrackSegment();
+    TrackSegment* ts = g_backend.allocSegment(L);
+    L->add(ts);
 
-    if (e.hasAttribute("xml:id"))
-        l->setId(e.attribute("xml:id"));
+    if (stream.attributes().hasAttribute("xml:id"))
+        ts->setId(IFeature::FId(IFeature::GpxSegment, stream.attributes().value("xml:id").toString().toLongLong()));
 
-    QDomElement c = e.firstChildElement();
-    while(!c.isNull()) {
-        if (c.tagName() == "trkpt") {
-            Node* N = Node::fromGPX(d, L, c);
-            l->add(N);
-            progress->setValue(progress->value()+1);
+    stream.readNext();
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == QStringLiteral("trkpt")) {
+            TrackNode* N = TrackNode::fromGPX(d, L, stream);
+            ts->add(N);
+            progress->setValue(stream.characterOffset());
         }
 
         if (progress->wasCanceled())
             break;
 
-        c = c.nextSiblingElement();
+        stream.readNext();
     }
 
-    return l;
+    return ts;
 }
 
-TrackSegment* TrackSegment::fromXML(Document* d, Layer* L, const QDomElement e, QProgressDialog * progress)
+TrackSegment* TrackSegment::fromXML(Document* d, Layer* L, QXmlStreamReader& stream, QProgressDialog * progress)
 {
-    return TrackSegment::fromGPX(d, L, e, progress);
+    return TrackSegment::fromGPX(d, L, stream, progress);
 }
