@@ -13,7 +13,7 @@ class CreateSingleWayInteraction : public FeatureSnapInteraction
     Q_OBJECT
 
     public:
-        CreateSingleWayInteraction(MainWindow* Main, MapView* aView, Node * firstNode, bool aCurved);
+        CreateSingleWayInteraction(MainWindow* aMain, Node * firstNode, bool aCurved);
         ~CreateSingleWayInteraction();
 
         virtual void snapMousePressEvent(QMouseEvent * event, Feature* aLast);
@@ -22,18 +22,17 @@ class CreateSingleWayInteraction : public FeatureSnapInteraction
         virtual void snapMouseDoubleClickEvent(QMouseEvent* , Feature*);
         virtual void paintEvent(QPaintEvent* anEvent, QPainter& thePainter);
         virtual QString toHtml();
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
         virtual QCursor cursor() const;
 #endif
 
 public:
-        void setSnapAngle(double angle);
+        void setSnapAngle(qreal angle);
         void setParallelMode(bool val);
 
         virtual void closeAndFinish();
 
     private:
-        MainWindow* Main;
         QPoint LastCursor;
         Way* theRoad;
         Coord FirstPoint;
@@ -42,7 +41,7 @@ public:
         bool Prepend;
         bool IsCurved;
         bool Creating;
-        double SnapAngle;
+        qreal SnapAngle;
         bool ParallelMode;
 };
 
