@@ -65,8 +65,8 @@ public:
     }
     ~ImageMapLayerPrivate()
     {
-        SAFE_DELETE(theMapAdapter)
-        SAFE_DELETE(theImageManager)
+        delete theMapAdapter;
+        delete theImageManager;
     }
 };
 
@@ -84,7 +84,7 @@ ImageMapLayer::ImageMapLayer(const QString & aName)
 
 ImageMapLayer::~ ImageMapLayer()
 {
-    SAFE_DELETE(p)
+    delete p;
 }
 
 CoordBox ImageMapLayer::boundingBox()
@@ -97,8 +97,8 @@ CoordBox ImageMapLayer::boundingBox()
 
     p->theProjection.setProjectionType(p->theMapAdapter->projection());
     QRectF r = p->theMapAdapter->getBoundingbox();
-    Coord tl = p->theProjection.inverse2Coord(r.topLeft());
-    Coord br = p->theProjection.inverse2Coord(r.bottomRight());
+    Coord tl = p->theProjection.inverse(r.topLeft());
+    Coord br = p->theProjection.inverse(r.bottomRight());
     return CoordBox(tl, br);
 }
 
@@ -154,7 +154,7 @@ QString ImageMapLayer::projection() const
     if (p->theMapAdapter)
         return p->theMapAdapter->projection();
 
-    return "";
+    return QString();
 }
 
 IImageManager* ImageMapLayer::getImageManger()
@@ -263,7 +263,7 @@ void ImageMapLayer::setMapAdapter(const QUuid& theAdapterUid, const QString& ser
                     QDialog dlg;
                     ui.setupUi(&dlg);
                     dlg.setWindowTitle(tr("Licensing Terms: %1").arg(name()));
-                    ui.webView->load(u);
+                    ui.urlLabel->setText(QString("<a href='%1'>%2</a>").arg(u.toString()).arg(u.toString()));
 
                     bool OK = false;
                     while (!OK) {
@@ -446,42 +446,42 @@ ImageMapLayer * ImageMapLayer::fromXML(Document* d, QXmlStreamReader& stream, QP
 
     qreal alpha = stream.attributes().value("alpha").toString().toDouble();
     // TODO: Note that the logic for "enabled" is slightly different. Why?
-    bool visible = (stream.attributes().value("visible") == "true" ? true : false);
-    bool selected = (stream.attributes().value("selected") == "true" ? true : false);
-    bool enabled = (stream.attributes().value("enabled") == "false" ? false : true);
+    bool visible = (stream.attributes().value("visible") == QStringLiteral("true") ? true : false);
+    bool selected = (stream.attributes().value("selected") == QStringLiteral("true") ? true : false);
+    bool enabled = (stream.attributes().value("enabled") == QStringLiteral("false") ? false : true);
 
     stream.readNext();
     while(!stream.atEnd() && !stream.isEndElement()) {
-        if (stream.name() == "AdjustmentList") {
+        if (stream.name() == QStringLiteral("AdjustmentList")) {
             stream.readNext();
             while(!stream.atEnd() && !stream.isEndElement()) {
-                if (stream.name() == "Adjustment") {
+                if (stream.name() == QStringLiteral("Adjustment")) {
                     int z = stream.attributes().value("zoom").toString().toInt();
                     if (l->p->AlignementTransformList.size() < z+1)
                         l->p->AlignementTransformList.resize(z+1);
                     l->p->AlignementTransformList[z] = QTransformFomXml(stream);
                     stream.readNext();
                 } else if (!stream.isWhitespace()) {
-                    qDebug() << "ImgLay: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+                    qDebug() << "ImgLay: logic error:" << stream.name() << ":" << stream.tokenType() << "(" << stream.lineNumber() << ")";
                     stream.skipCurrentElement();
                 }
                 stream.readNext();
             }
-        } else if (stream.name() == "WmsServer") {
+        } else if (stream.name() == QStringLiteral("WmsServer")) {
             server = stream.attributes().value("name").toString();
             l->setMapAdapter(bgtype, server);
             stream.readNext();
-        } else if (stream.name() == "TmsServer") {
+        } else if (stream.name() == QStringLiteral("TmsServer")) {
             server = stream.attributes().value("name").toString();
             l->setMapAdapter(bgtype, server);
             stream.readNext();
-        } else if (stream.name() == "Data") {
+        } else if (stream.name() == QStringLiteral("Data")) {
             l->setMapAdapter(bgtype, server);
             stream.readNext();
             if (l->getMapAdapter())
                 l->getMapAdapter()->fromXML(stream);
         } else if (!stream.isWhitespace()) {
-            qDebug() << "ImgLay: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+            qDebug() << "ImgLay: logic error:" << stream.name() << ":" << stream.tokenType() << "(" << stream.lineNumber() << ")";
             stream.skipCurrentElement();
         }
         stream.readNext();
@@ -644,7 +644,6 @@ void ImageMapLayer::draw(MapView& theView, QRect& rect)
 
     if (p->curPix.size() != rect.size()) {
         p->curPix = QPixmap(rect.size());
-        p->curPix.fill(Qt::transparent);
     }
 
     if (p->newPix.isNull())
@@ -653,10 +652,10 @@ void ImageMapLayer::draw(MapView& theView, QRect& rect)
     const QSize ps = p->pr.size();
     const QSize pmSize = p->newPix.size();
     const qreal ratio = qMax<const qreal>((qreal)pmSize.width()/ps.width()*1.0, (qreal)pmSize.height()/ps.height()*1.0);
-//    qDebug() << "Bg image ratio " << ratio;
+//    qDebug() << "Bg image ratio" << ratio;
     QPixmap pms;
     if (ratio >= 1.0) {
-//        qDebug() << "Bg image scale 1 " << ps << " : " << p->newPix.size();
+//        qDebug() << "Bg image scale 1" << ps << ":" << p->newPix.size();
         pms = p->newPix.scaled(ps);
     } else {
         const QSizeF drawingSize = pmSize * ratio;
@@ -664,13 +663,16 @@ void ImageMapLayer::draw(MapView& theView, QRect& rect)
         const QPointF drawingOrigin = QPointF(originSize.width(), originSize.height());
         const QRect drawingRect = QRect(drawingOrigin.toPoint(), drawingSize.toSize());
 
-//        qDebug() << "Bg image scale 2 " << ps << " : " << p->newPix.size();
+//        qDebug() << "Bg image scale 2" << ps << ":" << p->newPix.size();
         if (ps*ratio != drawingRect.size())
             pms = p->newPix.copy(drawingRect).scaled(ps*ratio);
         else
             pms = p->newPix.copy(drawingRect);
     }
 
+    /* We need to clear the pixmap in case we're painting transparent stuff over */
+    p->curPix.fill(Qt::transparent);
+
     QPainter P(&p->curPix);
     P.drawPixmap((pmSize.width()-pms.width())/2, (pmSize.height()-pms.height())/2, pms);
     //    if (p->theMapAdapter->isTiled())
@@ -689,8 +691,8 @@ QRect ImageMapLayer::drawFull(MapView& theView, QRect& rect)
 
     MapView::transformCalc(p->theTransform, p->theProjection, 0.0, CoordBox(alignedViewport), rect);
 
-    CoordBox cViewport(p->theProjection.inverse2Coord(p->theTransform.inverted().map(fRect.bottomLeft())),
-                     p->theProjection.inverse2Coord(p->theTransform.inverted().map(fRect.topRight())));
+    CoordBox cViewport(p->theProjection.inverse(p->theTransform.inverted().map(fRect.bottomLeft())),
+                     p->theProjection.inverse(p->theTransform.inverted().map(fRect.topRight())));
     CoordBox Viewport = CoordBox(p->AlignementTransformList.at(0).mapRect(cViewport));
     QPointF bl = theView.toView(Viewport.bottomLeft());
     QPointF tr = theView.toView(Viewport.topRight());
@@ -760,7 +762,7 @@ QRect ImageMapLayer::drawFull(MapView& theView, QRect& rect)
         } else if (p->theMapAdapter->getType() == IMapAdapter::NetworkDataBackground) {
 //            QString url (p->theMapAdapter->getQuery(wgs84vp, vp, rect));
 //            if (!url.isEmpty()) {
-//                qDebug() << "ImageMapLayer::drawFull: getting: " << url;
+//                qDebug() << "ImageMapLayer::drawFull: getting:" << url;
 //                QByteArray ba = p->theMapAdapter->getImageManager()->getData(p->theMapAdapter,url);
 //                QDomDocument* theXmlDoc = new QDomDocument();
 //                theXmlDoc->setContent(ba);
@@ -820,7 +822,7 @@ QRect ImageMapLayer::drawFull(MapView& theView, QRect& rect)
         } else if (p->theMapAdapter->getType() == IMapAdapter::NetworkBackground || p->theMapAdapter->getType() == IMapAdapter::BrowserBackground) {
             QString url (p->theMapAdapter->getQuery(wgs84vp, vp, rect));
             if (!url.isEmpty()) {
-                qDebug() << "ImageMapLayer::drawFull: getting: " << url;
+                //qDebug() << "ImageMapLayer::drawFull: getting:" << url;
                 QPixmap pm = QPixmap::fromImage(p->theMapAdapter->getImageManager()->getImage(p->theMapAdapter,url));
                 if (!pm.isNull()) {
                     p->curPix = QPixmap();
@@ -904,8 +906,8 @@ QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
     QPointF vp0Center = QPointF(projVp.width()/2, -projVp.height()/2);
 
     Coord ulCoord, lrCoord;
-    ulCoord = p->theProjection.inverse2Coord(vlm.topLeft());
-    lrCoord = p->theProjection.inverse2Coord(vlm.bottomRight());
+    ulCoord = p->theProjection.inverse(vlm.topLeft());
+    lrCoord = p->theProjection.inverse(vlm.bottomRight());
 
     const QPointF tl = theView.transform().map(theView.projection().project(ulCoord));
     const QPointF br = theView.transform().map(theView.projection().project(lrCoord));
@@ -916,11 +918,11 @@ QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
     QPointF vpCenter0 = QPointF(vpCenter.x()-p->theMapAdapter->getBoundingbox().left(), p->theMapAdapter->getBoundingbox().bottom()-vpCenter.y());
     qreal mapmiddle_tile_x = qRound(vpCenter0.x()/tileWidth);
     qreal mapmiddle_tile_y = qRound(vpCenter0.y()/tileHeight);
-    qDebug() << "z: " << p->theMapAdapter->getAdaptedZoom() << "; t_x: " << mapmiddle_tile_x << "; t_y: " << mapmiddle_tile_y ;
+    //qDebug() << "z:" << p->theMapAdapter->getAdaptedZoom() << "; t_x:" << mapmiddle_tile_x << "; t_y:" << mapmiddle_tile_y ;
 
     qreal cross_x = vpCenter0.x() - mapmiddle_tile_x*tileWidth;		// position on middle tile
     qreal cross_y = vpCenter0.y() - mapmiddle_tile_y*tileHeight;
-    qDebug() << "cross_x: " << cross_x << "; cross_y: " << cross_y;
+    //qDebug() << "cross_x:" << cross_x << "; cross_y:" << cross_y;
 
         // calculate how many surrounding tiles have to be drawn to fill the display
     qreal space_left = vp0Center.x() - cross_x;
@@ -947,14 +949,12 @@ QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
     int cross_scr_y = cross_y * tilesizeH / tileHeight;
 
     QSize pmSize = fRect.size().toSize();
-//    QSize pmSize((tiles_right+tiles_left+1)*tilesizeW, (tiles_bottom+tiles_above+1)*tilesizeH);
-//    QPixmap tmpPm = p->pm.scaled(retRect.size(), Qt::IgnoreAspectRatio);
     p->newPix = QPixmap(pmSize);
     p->newPix.fill(Qt::transparent);
     QPainter painter(&p->newPix);
 //    painter.drawPixmap(0, 0, tmpPm);
 
-//    qDebug() << "Tiles: " << tiles_right+tiles_left+1 << "x" << tiles_bottom+tiles_above+1;
+//    qDebug() << "Tiles:" << tiles_right+tiles_left+1 << "x" << tiles_bottom+tiles_above+1;
     for (i=-tiles_left; i<=tiles_right; i++)
     {
         for (j=-tiles_above; j<=tiles_bottom; j++)
@@ -971,30 +971,29 @@ QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
         }
     }
 
-    qSort(tiles);
+    // TODO: Sort the tiles? Removed when refactoring to qt6.
+    //qSort(tiles);
 
     int n=0; // Arbitrarily limit the number of tiles to 100
     for (QList<Tile>::const_iterator tile = tiles.begin(); tile != tiles.end() && n<100; ++tile)
     {
         QImage pm = p->theMapAdapter->getImageManager()->getImage(p->theMapAdapter, p->theMapAdapter->getQuery(mapmiddle_tile_x+tile->i, mapmiddle_tile_y+tile->j, p->theMapAdapter->getZoom()));
+        int x = (tile->i*tilesizeW)+pmSize.width()/2 -cross_scr_x;
+        int y = (tile->j*tilesizeH)+pmSize.height()/2-cross_scr_y;
         if (!pm.isNull())
-            painter.drawImage((tile->i*tilesizeW)+pmSize.width()/2 -cross_scr_x,
-                               (tile->j*tilesizeH)+pmSize.height()/2-cross_scr_y,
-                               pm);
+            painter.drawImage(x, y, pm);
+
+        if (M_PREFS->getDrawTileBoundary())
+            painter.drawRect(x, y, tilesizeW, tilesizeH);
 
-        if (M_PREFS->getDrawTileBoundary()) {
-            painter.drawRect((tile->i*tilesizeW)+pmSize.width()/2 -cross_scr_x,
-                             (tile->j*tilesizeH)+pmSize.height()/2-cross_scr_y,
-                             tilesizeW, tilesizeH);
-        }
         ++n;
     }
     painter.end();
 
-//    qDebug() << "tl: " << tl << "; br: " << br;
-//    qDebug() << "vp: " << projVp;
-    //    qDebug() << "vlm: " << vlm;
-    qDebug() << "retRect: " << retRect;
+//    qDebug() << "tl:" << tl << "; br:" << br;
+//    qDebug() << "vp:" << projVp;
+    //    qDebug() << "vlm:" << vlm;
+    //qDebug() << "retRect:" << retRect;
 //    QRect expR = QRect(-retRect.left(), -retRect.top(), retRect.width()+retRect.left(), retRect.height()+retRect.top());
 //    p->newPix.save("c:/tmp.png");
 //    p->newPix.copy(expR).save("c:/tmp2.png");
